package net.knarcraft.minstrel.command;

import net.citizensnpcs.api.CitizensAPI;
import net.citizensnpcs.api.npc.NPC;
import net.knarcraft.minstrel.trait.MinstrelTrait;
import org.bukkit.Sound;
import org.bukkit.SoundCategory;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.command.TabCompleter;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.List;

import static net.knarcraft.knarlib.util.TabCompletionHelper.filterMatchingContains;

/**
 * The tab-completer for the minstrel command
 */
public class MinstrelTabCompleter implements TabCompleter {

    private final List<String> baseCommands;
    private final List<String> soundCategories;
    private final List<String> exampleLengths;
    private final List<String> empty = new ArrayList<>();
    private final List<String> exampleFloats;
    private final List<String> booleans;

    /**
     * Instantiates a new minstrel tab completer
     */
    public MinstrelTabCompleter() {
        baseCommands = new ArrayList<>();
        baseCommands.add("addSong");
        baseCommands.add("removeSong");
        baseCommands.add("listSongs");
        baseCommands.add("pitch");
        baseCommands.add("volume");
        soundCategories = new ArrayList<>();
        for (SoundCategory category : SoundCategory.values()) {
            soundCategories.add(category.name());
        }
        exampleLengths = new ArrayList<>();
        exampleLengths.add("60");
        exampleLengths.add("90");
        exampleLengths.add("120");
        exampleFloats = new ArrayList<>();
        exampleFloats.add("0.5");
        exampleFloats.add("1");
        exampleFloats.add("1.5");
        booleans = new ArrayList<>();
        booleans.add("true");
        booleans.add("false");
    }

    @Nullable
    @Override
    public List<String> onTabComplete(@NotNull CommandSender sender, @NotNull Command command, @NotNull String label,
                                      @NotNull String[] args) {
        NPC npc = CitizensAPI.getDefaultNPCSelector().getSelected(sender);
        if (npc == null || !npc.hasTrait(MinstrelTrait.class)) {
            return empty;
        }
        MinstrelTrait minstrelTrait = npc.getTraitNullable(MinstrelTrait.class);

        if (args.length < 2) {
            return filterMatchingContains(baseCommands, args[0]);
        }

        switch (args[0].toLowerCase()) {
            case "addsong":
                return tabCompleteAddSong(args);
            case "removesong":
                return tabCompleteRemoveSong(minstrelTrait, args);
            case "listsongs":
                return empty;
            case "volume":
            case "pitch":
                if (args.length == 2) {
                    return exampleFloats;
                } else if (args.length == 3) {
                    return booleans;
                } else {
                    return empty;
                }
        }
        return null;
    }

    /**
     * Gets the tab-completions for removing a song
     *
     * @param minstrelTrait <p>The currently selected minstrel</p>
     * @param arguments     <p>THe arguments given by the user</p>
     * @return <p>The tab-completion options to display to the user</p>
     */
    @NotNull
    private List<String> tabCompleteRemoveSong(@NotNull MinstrelTrait minstrelTrait, @NotNull String[] arguments) {
        if (arguments.length == 2) {
            List<String> indices = new ArrayList<>();
            for (int i = 0; i < minstrelTrait.getPlaylist().getSongs().size(); i++) {
                indices.add(String.valueOf(i));
            }
            return filterMatchingContains(indices, arguments[1]);
        } else {
            return empty;
        }
    }

    /**
     * Gets the tab-completions for adding a song
     *
     * @param arguments <p>The arguments given by the user</p>
     * @return <p>The tab-completion options to display to the user</p>
     */
    @NotNull
    private List<String> tabCompleteAddSong(@NotNull String[] arguments) {
        if (arguments.length == 2) {
            return filterMatchingContains(soundCategories, arguments[1]);
        } else if (arguments.length == 3) {
            List<String> exampleSongNames = new ArrayList<>();
            for (Sound sound : Sound.values()) {
                exampleSongNames.add(sound.getKey().getNamespace() + ":" + sound.getKey().getKey());
            }
            return filterMatchingContains(exampleSongNames, arguments[2]);
        } else if (arguments.length == 4) {
            return filterMatchingContains(exampleLengths, arguments[3]);
        } else {
            return empty;
        }
    }

}
