package net.knarcraft.minstrel.manager;

import net.knarcraft.minstrel.music.SongEndTime;
import net.knarcraft.minstrel.trait.MinstrelTrait;
import net.knarcraft.minstrel.util.SoundHelper;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.PriorityQueue;
import java.util.Queue;
import java.util.Set;

/**
 * A manager keeping track of the queue for playing the next song
 */
public class QueueManager {

    private static final Queue<SongEndTime> songEndTimes = new PriorityQueue<>();
    private static final Map<Player, Set<MinstrelTrait>> queuedMinstrels = new HashMap<>();

    /**
     * Takes care of any expired songs
     */
    public static void handleQueue() {
        while (!songEndTimes.isEmpty() && songEndTimes.peek().endTime() < System.currentTimeMillis()) {
            SongEndTime songEndTime = songEndTimes.remove();
            if (songEndTime == null) {
                continue;
            }

            // Play the next song if the player is near enough to hear
            MinstrelTrait minstrelTrait = songEndTime.minstrelTrait();
            Player player = songEndTime.player();
            if (SoundHelper.canHear(player, minstrelTrait)) {
                minstrelTrait.getPlaylist().play(minstrelTrait, player);
            } else {
                queuedMinstrels.get(player).remove(minstrelTrait);
            }
        }
    }

    /**
     * Checks whether the given minstrel has a song queued for the given player
     *
     * @param player        <p>The player to check</p>
     * @param minstrelTrait <p>The minstrel to check</p>
     * @return <p>True if already queued</p>
     */
    public static boolean isQueued(@NotNull Player player, @NotNull MinstrelTrait minstrelTrait) {
        return queuedMinstrels.get(player) != null && queuedMinstrels.get(player).contains(minstrelTrait);
    }

    /**
     * Queues the given song end time
     *
     * @param songEndTime <p>The song end time to queue</p>
     */
    public static void queue(@NotNull SongEndTime songEndTime) {
        songEndTimes.add(songEndTime);
        queuedMinstrels.computeIfAbsent(songEndTime.player(), k -> new HashSet<>());
        queuedMinstrels.get(songEndTime.player()).add(songEndTime.minstrelTrait());
    }

    /**
     * Clears the song queue for the given player
     *
     * @param player <p>The player to clear the song queue for</p>
     */
    public static void clearQueue(@NotNull Player player) {
        queuedMinstrels.remove(player);
        songEndTimes.removeIf((songEndTime -> songEndTime.player().equals(player)));
    }

}
