package net.knarcraft.minstrel.music;

import net.knarcraft.minstrel.manager.QueueManager;
import net.knarcraft.minstrel.trait.MinstrelTrait;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * A representation of a playlist containing songs
 */
public class Playlist {

    private final List<Song> songs;
    private final Map<Player, Integer> playerCurrentSong = new HashMap<>();

    /**
     * Instantiates a new playlist
     *
     * @param songs <p>The songs contained in this playlist</p>
     */
    public Playlist(@NotNull List<Song> songs) {
        this.songs = new ArrayList<>(songs);
    }

    /**
     * Gets a copy of the songs in this playlist
     *
     * @return <p>The songs in this playlist</p>
     */
    @NotNull
    public List<Song> getSongs() {
        return new ArrayList<>(this.songs);
    }

    /**
     * Adds a new song to this playlist
     *
     * @param song <p>The song to add</p>
     */
    public void addSong(@NotNull Song song) {
        this.songs.add(song);
    }

    /**
     * Removes a song from this playlist
     *
     * @param index <p>The index of the song to remove</p>
     */
    public void removeSong(int index) {
        this.songs.remove(index);
    }

    /**
     * Stops all songs in this playlist for the given player
     *
     * @param player <p>The player to stop the playlist for</p>
     */
    public void stop(@NotNull Player player) {
        Integer currentSong = this.playerCurrentSong.get(player);
        if (currentSong == null) {
            return;
        }
        Song currentlyPlaying = this.songs.get(currentSong);
        if (currentlyPlaying != null) {
            currentlyPlaying.stop(player);
            playerCurrentSong.remove(player);
        }
    }

    /**
     * Stops all songs in this playlist for all players, and aborts scheduling for the next song
     *
     * <p>This should only be triggered if a Minstrel, or a playlist song, is removed.</p>
     */
    public void stop() {
        for (Player player : Bukkit.getOnlinePlayers()) {
            for (Song song : this.getSongs()) {
                song.stop(player);
            }
        }
    }

    /**
     * Plays the current song for the given player
     *
     * <p>This should only be used if a player recently joined, or for some other reason cannot currently hear the
     * music.</p>
     *
     * @param trait  <p>The the minstrel to play this playlist for</p>
     * @param player <p>The player to play to</p>
     */
    public void play(@NotNull MinstrelTrait trait, @NotNull Player player) {
        //If this playlist is empty, do nothing
        if (this.songs.isEmpty()) {
            return;
        }

        Song currentSong;
        if (this.playerCurrentSong.containsKey(player)) {
            int songId = (this.playerCurrentSong.get(player) + 1) % songs.size();
            this.playerCurrentSong.put(player, songId);
            currentSong = songs.get(songId);
        } else {
            this.playerCurrentSong.put(player, 0);
            currentSong = songs.get(0);
        }
        currentSong.play(trait, player, trait.getVolume(), trait.getPitch());
        QueueManager.queue(new SongEndTime(player, trait, System.currentTimeMillis() +
                (currentSong.getDuration() * 1000L)));
    }

    /**
     * Clears this playlist, removing any existing songs
     */
    public void clear() {
        this.songs.clear();
    }

    @Override
    @NotNull
    public String toString() {
        StringBuilder builder = new StringBuilder();
        for (Song song : this.songs) {
            builder.append("\n");
            builder.append(song);
        }
        return builder.toString();
    }

}
